@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  *
- $Id: acpi-linux.c,v 1.12 2005/01/14 04:06:40 ryanl Exp $
+ $Id: acpi-linux.c,v 1.14 2005/07/04 14:15:17 ryanl Exp $
  */
 
 /*
@@ -147,6 +147,7 @@ static gboolean update_ac_info(struct acpi_info * acpiinfo)
   struct dirent * procdirentry;
   char buf[BUFSIZ];
   GHashTable *hash;
+  gboolean have_adaptor = FALSE;
 
   acpiinfo->ac_online = FALSE;
 
@@ -158,6 +159,7 @@ static gboolean update_ac_info(struct acpi_info * acpiinfo)
    {
     if (procdirentry->d_name[0]!='.')
      {
+      have_adaptor = TRUE;
       strcpy(ac_state,"/proc/acpi/ac_adapter/");
       strcat(ac_state,procdirentry->d_name);
       strcat(ac_state,"/");
@@ -172,6 +174,13 @@ static gboolean update_ac_info(struct acpi_info * acpiinfo)
        }
      }
    }
+
+  /* If there are no AC adaptors registered in the system, then we're
+     probably on a desktop (and therefore, on AC power).
+   */
+  if (have_adaptor == FALSE)
+    acpiinfo->ac_online = 1;
+
   closedir(procdir);
 
   return TRUE;
@@ -329,6 +338,10 @@ gboolean acpi_process_event(struct acpi_info * acpiinfo)
           break;
         case ACPI_EVENT_BATTERY_INFO:
           update_battery_info(acpiinfo);
+          /* Update AC info on battery info updates.  This works around
+           * a bug in ACPI (as per bug #163013).
+           */
+          update_ac_info(acpiinfo);
           result = TRUE;
           break;
       }
