@@ -3,6 +3,7 @@
  */
 
 #include <config.h>
+#include <string.h>
 #include <panel-applet.h>
 #include <libgnomeui/gnome-help.h>
 #ifdef HAVE_GUCHARMAP
@@ -10,76 +11,74 @@
 #endif
 #include "charpick.h"
 
-
-
 /* The comment for each char list has the html entity names of the chars */
-/* All gunicar codes should end in -1 */
+/* All gunicar codes should end in 0 */
 
 
 /* This is the default list used when starting charpick the first time */
 /* aacute, agrave, eacute, iacute, oacute, frac12, copy*/
 /* static const gchar *def_list = "áàéíñóœ©"; */
-static gunichar def_code[] = {225, 224, 233, 237, 241, 243, 189, 169, 1579, 8364, -1};
+static const gunichar def_code[] = {225, 224, 233, 237, 241, 243, 189, 169, 1579, 8364, 0};
 
 /* aacute, agrave, acirc, atilde, auml. aring, aelig, ordf */
 /* static const gchar *a_list = "áàâãäåæª"; */
-static gunichar a_code[] = {225, 224, 226, 227, 228, 229, 230, 170, -1};
+static const gunichar a_code[] = {225, 224, 226, 227, 228, 229, 230, 170, 0};
 /* static const gchar *cap_a_list = "ÁÀÂÃÄÅÆª"; */
-static gunichar cap_a_code[] = {192, 193, 194, 195, 196, 197, 198, 170, -1}; 
+static const gunichar cap_a_code[] = {192, 193, 194, 195, 196, 197, 198, 170, 0}; 
 /* ccedil, cent, copy */
 /* static const gchar *c_list = "çÇ¢©"; */
-static gunichar c_code[] = {231, 199, 162, 169, -1};
+static const gunichar c_code[] = {231, 199, 162, 169, 0};
 /* eacute, egrave, ecirc, euml, aelig */
 /* static const gchar *e_list = "éèêëæ"; */
-static gunichar e_code[] = {232, 233, 234, 235, 230, -1};
+static const gunichar e_code[] = {232, 233, 234, 235, 230, 0};
 /* static const gchar *cap_e_list = "ÉÈÊËÆ"; */
-static gunichar cap_e_code[] = {200, 201, 202, 203, 198, -1};
+static const gunichar cap_e_code[] = {200, 201, 202, 203, 198, 0};
 /* iacute, igrave, icirc, iuml */
 /* static const gchar *i_list = "íìîï"; */
-static gunichar i_code[] = {236, 237, 238, 239, -1};
+static const gunichar i_code[] = {236, 237, 238, 239, 0};
 /* static const gchar *cap_i_list = "ÍÌÎÏ"; */
-static gunichar cap_i_code[] = {204, 205, 206, 207, -1};
+static const gunichar cap_i_code[] = {204, 205, 206, 207, 0};
 /* ntilde (this is the most important line in this program.) */
 /* static const gchar *n_list = "ñ, Ñ"; */
-static gunichar n_code[] = {241, 209, -1};
+static const gunichar n_code[] = {241, 209, 0};
 /* oacute, ograve, ocirc, otilde, ouml, oslash, ordm */
 /* static const gchar *o_list = "óòôõöøº"; */
-static gunichar o_code[] = {242, 243, 244, 245, 246, 248, 176, -1};
+static const gunichar o_code[] = {242, 243, 244, 245, 246, 248, 176, 0};
 /* static const gchar *cap_o_list = "ÓÒÔÕÖØº"; */
-static gunichar cap_o_code[] = {210, 211, 212, 213, 214, 216, 176, -1};
+static const gunichar cap_o_code[] = {210, 211, 212, 213, 214, 216, 176, 0};
 /* szlig, sect, dollar */
 /* static const gchar *s_list = "ß§$"; */
-static gunichar s_code[] = {223, 167, 36, -1};
+static const gunichar s_code[] = {223, 167, 36, 0};
 /* eth, thorn */
 /* static const gchar *t_list = "ðÐþÞ"; */
-static gunichar t_code[] = {240, 208, 254, 222, -1};
+static const gunichar t_code[] = {240, 208, 254, 222, 0};
 /* uacute, ugrave, ucirc, uuml */
 /* static const gchar *u_list = "úùûü"; */
-static gunichar u_code[] = {249, 250, 251, 252, -1};
+static const gunichar u_code[] = {249, 250, 251, 252, 0};
 /* static const gchar *cap_u_list = "ÚÙÛÜ"; */
-static gunichar cap_u_code[] = {217, 218, 219, 220, -1};
+static const gunichar cap_u_code[] = {217, 218, 219, 220, 0};
 /* yacute, yuml, yen Yes, there is no capital yuml in iso 8859-1.*/
 /* static const gchar *y_list = "ýÝÿ¥"; */
-static gunichar y_code[] = {253, 221, 255, 165, -1};
+static const gunichar y_code[] = {253, 221, 255, 165, 0};
 
 /* extra characters unrelated to the latin alphabet. All characters in 
    ISO-8859-1 should now be accounted for.*/
 /* not shy macr plusmn */
 /* static const gchar *dash_list = "¬­¯±"; */
-static gunichar dash_code[] = {172, 173, 175, 177, -1};
+static const gunichar dash_code[] = {172, 173, 175, 177, 0};
 /* laquo raquo uml */
 /* static const gchar *quote_list = "«»š·×"; */
-static gunichar quote_code[] = {171, 187, 168, 183, 215, -1};
+static const gunichar quote_code[] = {171, 187, 168, 183, 215, 0};
 /* curren, pound, yen, cent, dollar */
 /* static const gchar *currency_list = "€£¥¢$"; */
-static gunichar currency_code[] = {164, 163, 165, 162, 36, 8364, -1}; 
+static const gunichar currency_code[] = {164, 163, 165, 162, 36, 8364, 0}; 
 /* sup1 frac12 */
 /* static const gchar *one_list = "¹œŒ"; */
-static gunichar one_code[] = {185, 178, 179, 188, 189, 190, -1};
+static const gunichar one_code[] = {185, 178, 179, 188, 189, 190, 0};
 /* µ ¶ ® ¿ ¡ |  */
-static gunichar misc_code[] = {181, 182, 174, 191, 161, 124, -1};
+static const gunichar misc_code[] = {181, 182, 174, 191, 161, 124, 0};
 
-static const gunichar *chartable[] = {
+static const gunichar * const chartable[] = {
 	def_code,
 	a_code,
 	cap_a_code,
@@ -148,8 +147,6 @@ selection_clear_cb (GtkWidget *widget, GdkEventSelection *event,
 {
   charpick_data *curr_data = data;
   
-  gint last_index = curr_data->last_index;
-  
   if (curr_data->last_toggle_button)
     gtk_toggle_button_set_state (curr_data->last_toggle_button, FALSE);
 
@@ -157,31 +154,12 @@ selection_clear_cb (GtkWidget *widget, GdkEventSelection *event,
   return TRUE;
 }
 
-static gchar *
-get_utf_string (gunichar *codes)
-{
-	gchar *string = NULL, tmp[7];
-	gint i = 0;
-	while (codes[i] != -1) {
-		gint num;
-		num = g_unichar_to_utf8 (codes[i], tmp);
-		tmp[num] = 0;
-		if (string) 
-			string = g_strconcat (string, tmp, NULL);
-		else
-			string = g_strdup (tmp);
-		i++;
-	}
-	return string;
-}
 
 static gint
 toggle_button_toggled_cb(GtkToggleButton *button, gpointer data)
 {
   charpick_data *curr_data = data;
-  GtkClipboard *clipboard;
   gint button_index;
-  gint last_index = curr_data->last_index;
   gboolean toggled;
    
   button_index = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (button), "index")); 
@@ -232,7 +210,7 @@ static gint
 key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
   charpick_data *p_curr_data = data;
-  gint *code = NULL;
+  const gunichar *code = NULL;
   gchar inputchar = event->keyval;
 
   switch (inputchar)
@@ -287,8 +265,9 @@ key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer data)
     		return FALSE;
     }
 #if 0
+  /* FIXME: what's wrong here ? */
   if (code)
-    p_curr_data->charlist = get_utf_string (code);
+    p_curr_data->charlist = g_ucs4_to_utf8 (code, -1, NULL, NULL, NULL);
   else
     p_curr_data->charlist = "hello";
   p_curr_data->last_index = NO_LAST_INDEX;
@@ -341,13 +320,14 @@ populate_menu (charpick_data *curr_data)
 			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem), TRUE);
 		list = g_list_next (list);
 	}
+	build_table(curr_data);
 }
 
 static void
 get_menu_pos (GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer data)
 {
 	charpick_data *curr_data = data;
-	GtkRequisition  reqmenu, reqapplet;
+	GtkRequisition  reqmenu;
 	gint tempx, tempy, width, height;
 	gint screen_width, screen_height;
 	
@@ -423,11 +403,9 @@ build_table(charpick_data *p_curr_data)
   GtkWidget *box, *button_box, **row_box;
   GtkWidget *button, *arrow;
   GtkTooltips *tooltips;
-  gint size;
-  gint i = 0, j, len = g_utf8_strlen (p_curr_data->charlist, -1);
+  gint i = 0, len = g_utf8_strlen (p_curr_data->charlist, -1);
   GtkWidget *toggle_button[len];
   gchar *charlist;
-  gint width, height;
   gint max_width=1, max_height=1;
   gint size_ratio;
 
@@ -444,36 +422,43 @@ build_table(charpick_data *p_curr_data)
   
   tooltips = gtk_tooltips_new ();  
   button = gtk_button_new ();
-  gtk_tooltips_set_tip (tooltips, button, _("Available palettes"), NULL);
-
-  switch (panel_applet_get_orient (PANEL_APPLET (p_curr_data->applet))) {
-     	case PANEL_APPLET_ORIENT_DOWN:
-        	arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT);
-     		break;
-     	case PANEL_APPLET_ORIENT_UP:
-        	arrow = gtk_arrow_new (GTK_ARROW_UP, GTK_SHADOW_OUT);  
-     		break;
-     	case PANEL_APPLET_ORIENT_LEFT:
-     		arrow = gtk_arrow_new (GTK_ARROW_LEFT, GTK_SHADOW_OUT);  
-		break;
-     	case PANEL_APPLET_ORIENT_RIGHT:
-     		arrow = gtk_arrow_new (GTK_ARROW_RIGHT, GTK_SHADOW_OUT);  
-		break;
+  if (g_list_length (p_curr_data->chartable) != 1)
+  {
+    gtk_tooltips_set_tip (tooltips, button, _("Available palettes"), NULL);
+  
+    switch (panel_applet_get_orient (PANEL_APPLET (p_curr_data->applet))) {
+       	case PANEL_APPLET_ORIENT_DOWN:
+          	arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT);
+       		break;
+       	case PANEL_APPLET_ORIENT_UP:
+          	arrow = gtk_arrow_new (GTK_ARROW_UP, GTK_SHADOW_OUT);  
+       		break;
+       	case PANEL_APPLET_ORIENT_LEFT:
+       		arrow = gtk_arrow_new (GTK_ARROW_LEFT, GTK_SHADOW_OUT);  
+  		break;
+       	case PANEL_APPLET_ORIENT_RIGHT:
+       		arrow = gtk_arrow_new (GTK_ARROW_RIGHT, GTK_SHADOW_OUT);  
+  		break;
+    default:
+  	  g_assert_not_reached ();
+    }
+    gtk_container_add (GTK_CONTAINER (button), arrow);
+    gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+    /* FIXME : evil hack (see force_no_focus_padding) */
+    force_no_focus_padding (button);
+    gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 0);
+    g_signal_connect (G_OBJECT (button), "clicked",
+                              G_CALLBACK (chooser_button_clicked),
+			      p_curr_data);
+    g_signal_connect (G_OBJECT (button), "button_press_event",
+                               G_CALLBACK (button_press_hack),
+			       p_curr_data->applet);
+  
   }
-  gtk_container_add (GTK_CONTAINER (button), arrow);
-  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-  /* FIXME : evil hack (see force_no_focus_padding) */
-  force_no_focus_padding (button);
-  gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 0);
-  g_signal_connect (G_OBJECT (button), "clicked",
-                            G_CALLBACK (chooser_button_clicked), p_curr_data);
-  g_signal_connect (G_OBJECT (button), "button_press_event",
-                             G_CALLBACK (button_press_hack), p_curr_data->applet);
-
+  
   charlist = g_strdup (p_curr_data->charlist);
   for (i = 0; i < len; i++) {
     gchar label[7];
-    gint num;
     GtkRequisition req;
     gchar atk_desc[50];
     gchar *name;
@@ -586,26 +571,18 @@ about (BonoboUIComponent *uic,
        charpick_data     *curr_data,
        const char        *verb)
 {
-  GdkPixbuf   	   *pixbuf;
-  GError      	   *error     = NULL;
-  gchar            *file;
-   
-  const char *authors[] = {
-	  /* If your charset supports it, please use U00F1 to replace the "n"
-	   * in "Muniz". */
-	  _("Alexandre Muniz <munizao@xprt.net>"),
+  static const char * const authors[] = {
+	  "Alexandre Muñiz <munizao@xprt.net>",
 	  "Kevin Vandersloot",
 	  NULL
   };
 
-  const gchar *documenters[] = {
+  static const gchar * const documenters[] = {
           "Dan Mueth <d-mueth@uchicago.edu>",
           "Sun GNOME Documentation Team <gdocteam@sun.com>",
 	  NULL
   };
 
-  const gchar *translator_credits = _("translator_credits");
-
   gtk_show_about_dialog (NULL,
 	"name",		_("Character Palette"),
 	"version",	VERSION,
@@ -645,7 +622,6 @@ static void
 applet_destroy (GtkWidget *widget, gpointer data)
 {
   charpick_data *curr_data = data;
-  GtkWidget *applet = curr_data->applet;
 
   g_return_if_fail (curr_data);
    
@@ -710,7 +686,7 @@ get_chartable (charpick_data *curr_data)
 		for (i=0; i<n; i++) {
 			gchar *string;
 		
-			string = get_utf_string ((gunichar *)chartable[i]);
+			string = g_ucs4_to_utf8 (chartable[i], -1, NULL, NULL, NULL);
 			curr_data->chartable = g_list_append (curr_data->chartable, string);
 		
 		}
