@@ -19,17 +19,47 @@
 
 #include <config.h>
 #include <libxml/parser.h>
-
+#include <X11/Xatom.h>
 #include <gdk/gdkx.h>
 #define WNCK_I_KNOW_THIS_IS_UNSTABLE 1
 #include <libwnck/libwnck.h>
+#include <string.h>
 
 #include "stickynotes.h"
 #include "stickynotes_callbacks.h"
 #include "util.h"
+#include "stickynotes_applet.h"
+
+/* Stop gcc complaining about xmlChar's signedness */
+#define XML_CHAR(str) ((xmlChar *) (str))
 
 static void response_cb (GtkWidget *dialog, gint id, gpointer data);
 
+/* Based on a function found in wnck */
+static void
+set_icon_geometry  (GdkWindow *window,
+                  int        x,
+                  int        y,
+                  int        width,
+                  int        height)
+{
+      gulong data[4];
+      Display *dpy = gdk_x11_drawable_get_xdisplay (window);
+
+      data[0] = x;
+      data[1] = y;
+      data[2] = width;
+      data[3] = height;
+
+      XChangeProperty (dpy,
+                       GDK_WINDOW_XID (window),
+                       gdk_x11_get_xatom_by_name_for_display (
+			       gdk_drawable_get_display (window),
+			       "_NET_WM_ICON_GEOMETRY"),
+		       XA_CARDINAL, 32, PropModeReplace,
+                       (guchar *)&data, 4);
+}
+
 /* Create a new (empty) Sticky Note */
 StickyNote *
 stickynote_new (GdkScreen *screen)
@@ -51,6 +81,9 @@ stickynote_new (GdkScreen *screen)
 
 	note->w_window = glade_xml_get_widget (window, "stickynote_window");
 	gtk_window_set_screen(GTK_WINDOW(note->w_window),screen);
+	gtk_window_set_decorated (GTK_WINDOW (note->w_window), FALSE);
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (note->w_window), TRUE);
+	gtk_window_set_skip_pager_hint (GTK_WINDOW (note->w_window), TRUE);
 
 	note->w_title = glade_xml_get_widget(window, "title_label");
 	note->w_body = glade_xml_get_widget(window, "body_text");
@@ -195,6 +228,8 @@ stickynote_new (GdkScreen *screen)
 			G_CALLBACK (properties_apply_font_cb), note);
 	g_signal_connect (G_OBJECT (note->w_entry), "activate",
 			G_CALLBACK (properties_activate_cb), note);
+	g_signal_connect (G_OBJECT (note->w_properties), "delete-event",
+			G_CALLBACK (gtk_widget_hide), note);
 
 	/* unref GladeXML objects */
 	g_object_unref(properties);
@@ -517,13 +552,7 @@ stickynote_set_visible (StickyNote *note, gboolean visible)
 {
 	if (visible)
 	{
-		gtk_window_set_decorated (GTK_WINDOW (note->w_window), FALSE);
-		gtk_window_set_skip_taskbar_hint (GTK_WINDOW (note->w_window),
-				TRUE);
-		gtk_window_set_skip_pager_hint (GTK_WINDOW (note->w_window),
-				TRUE);
-		gtk_window_set_keep_above (GTK_WINDOW (note->w_window), TRUE);
-		gtk_widget_show (note->w_window);
+		gtk_window_present (GTK_WINDOW (note->w_window));
 		
 		if (note->x != -1 || note->y != -1)
 			gtk_window_move (GTK_WINDOW (note->w_window),
@@ -561,7 +590,11 @@ stickynote_set_visible (StickyNote *note, gboolean visible)
 	}
 	else {
 		/* Hide sticky note */
-		gtk_widget_hide(note->w_window);
+		int x, y, width, height;
+		stickynotes_applet_panel_icon_get_geometry (&x, &y, &width, &height);
+		set_icon_geometry (GTK_WIDGET (note->w_window)->window,
+				   x, y, width, height);
+		gtk_window_iconify(GTK_WINDOW (note->w_window));
 	}
 }
 
@@ -575,8 +608,7 @@ void stickynotes_add (GdkScreen *screen)
 	stickynotes->notes = g_list_append(stickynotes->notes, note);
 	stickynotes_applet_update_tooltips();
 	stickynotes_save();
-	stickynote_set_visible (note, gconf_client_get_bool (stickynotes->gconf,
-				GCONF_PATH "/settings/visible", NULL));
+	stickynote_set_visible (note, TRUE);
 }
 
 /* Remove a sticky note with confirmation, if needed */
@@ -625,11 +657,11 @@ stickynotes_save (void)
 	gint i;
 	
 	/* Create a new XML document */
-	xmlDocPtr doc = xmlNewDoc("1.0");
-	xmlNodePtr root = xmlNewDocNode(doc, NULL, "stickynotes", NULL);
+	xmlDocPtr doc = xmlNewDoc(XML_CHAR ("1.0"));
+	xmlNodePtr root = xmlNewDocNode(doc, NULL, XML_CHAR ("stickynotes"), NULL);
 
 	xmlDocSetRootElement(doc, root);
-	xmlNewProp(root, "version", VERSION);
+	xmlNewProp(root, XML_CHAR("version"), XML_CHAR (VERSION));
 
 	wnck_screen = wnck_screen_get_default ();
 	wnck_screen_force_update (wnck_screen);
@@ -652,18 +684,15 @@ stickynotes_save (void)
 
 		xid = GDK_WINDOW_XID (note->w_window->window);
 		wnck_win = wnck_window_get (xid);
-		if (gconf_client_get_bool (stickynotes->gconf,
-			     		GCONF_PATH "/settings/visible", NULL))
-		{
-			if (!gconf_client_get_bool (stickynotes->gconf,
-					GCONF_PATH "/settings/sticky", NULL) &&
-				wnck_win)
-				note->workspace = 1 +
-					wnck_workspace_get_number (
-					wnck_window_get_workspace (wnck_win));
-			else
-				note->workspace = 0;
-		}
+
+		if (!gconf_client_get_bool (stickynotes->gconf,
+				GCONF_PATH "/settings/sticky", NULL) &&
+			wnck_win)
+			note->workspace = 1 +
+				wnck_workspace_get_number (
+				wnck_window_get_workspace (wnck_win));
+		else
+			note->workspace = 0;
 		
 		/* Retreive the title of the note */
 		title = gtk_label_get_text(GTK_LABEL(note->w_title));
@@ -676,29 +705,29 @@ stickynotes_save (void)
 
 		/* Save the note as a node in the XML document */
 		{
-			xmlNodePtr node = xmlNewTextChild(root, NULL, "note",
-					body);		
-			xmlNewProp(node, "title", title);
+			xmlNodePtr node = xmlNewTextChild(root, NULL, XML_CHAR ("note"),
+					XML_CHAR (body));		
+			xmlNewProp(node, XML_CHAR ("title"), XML_CHAR (title));
 			if (note->color)
-				xmlNewProp (node, "color", note->color);
+				xmlNewProp (node, XML_CHAR ("color"), XML_CHAR (note->color));
 			if (note->font_color)
-				xmlNewProp (node, "font_color",
-						note->font_color);
+				xmlNewProp (node, XML_CHAR ("font_color"),
+						XML_CHAR (note->font_color));
 			if (note->font)
-				xmlNewProp (node, "font", note->font);
+				xmlNewProp (node, XML_CHAR ("font"), XML_CHAR (note->font));
 			if (note->locked)
-				xmlNewProp (node, "locked", "true");
-			xmlNewProp (node, "x", x_str);
-			xmlNewProp (node, "y", y_str);
-			xmlNewProp (node, "w", w_str);
-			xmlNewProp (node, "h", h_str);
+				xmlNewProp (node, XML_CHAR ("locked"), XML_CHAR ("true"));
+			xmlNewProp (node, XML_CHAR ("x"), XML_CHAR (x_str));
+			xmlNewProp (node, XML_CHAR ("y"), XML_CHAR (y_str));
+			xmlNewProp (node, XML_CHAR ("w"), XML_CHAR (w_str));
+			xmlNewProp (node, XML_CHAR ("h"), XML_CHAR (h_str));
 			if (note->workspace > 0)
 			{
 				char *workspace_str;
 				
 				workspace_str = g_strdup_printf ("%i",
 						note->workspace);
-				xmlNewProp (node, "workspace", workspace_str);
+				xmlNewProp (node, XML_CHAR ("workspace"), XML_CHAR (workspace_str));
 				g_free (workspace_str);
 			}
 		}
@@ -752,7 +781,7 @@ stickynotes_load (GdkScreen *screen)
 	
 	/* If the XML file is corrupted/incorrect, create a blank one */
 	root = xmlDocGetRootElement(doc);
-	if (!root || xmlStrcmp(root->name, (const xmlChar *) "stickynotes"))
+	if (!root || xmlStrcmp(root->name, XML_CHAR ("stickynotes")))
 	{
 		xmlFreeDoc(doc);
 		stickynotes_save();
@@ -776,7 +805,7 @@ stickynotes_load (GdkScreen *screen)
 
 			/* Retrieve and set title of the note */
 			{
-				gchar *title = xmlGetProp(node, "title");
+				gchar *title = (gchar *)xmlGetProp(node, XML_CHAR ("title"));
 				if (title)
 					stickynote_set_title (note, title);
 				g_free (title);
@@ -784,12 +813,11 @@ stickynotes_load (GdkScreen *screen)
 
 			/* Retrieve and set the color of the note */
 			{
-				char *color_str;
-				char *font_color_str;
+				gchar *color_str;
+				gchar *font_color_str;
 				
-				color_str = xmlGetProp (node, "color");
-				font_color_str = xmlGetProp (node,
-						"font_color");
+				color_str = (gchar *)xmlGetProp (node, XML_CHAR ("color"));
+				font_color_str = (gchar *)xmlGetProp (node, XML_CHAR ("font_color"));
 				
 				if (color_str || font_color_str)
 					stickynote_set_color (note,
@@ -802,7 +830,7 @@ stickynotes_load (GdkScreen *screen)
 
 			/* Retrieve and set the font of the note */
 			{
-				gchar *font_str = xmlGetProp(node, "font");
+				gchar *font_str = (gchar *)xmlGetProp(node, XML_CHAR ("font"));
 				if (font_str)
 					stickynote_set_font (note, font_str,
 							TRUE);
@@ -811,8 +839,8 @@ stickynotes_load (GdkScreen *screen)
 
 			/* Retrieve and set the window size of the note */
 			{
-				gchar *w_str = xmlGetProp(node, "w");
-				gchar *h_str = xmlGetProp(node, "h");
+				gchar *w_str = (gchar *)xmlGetProp(node, XML_CHAR ("w"));
+				gchar *h_str = (gchar *)xmlGetProp(node, XML_CHAR ("h"));
 				if (w_str && h_str)
 					gtk_window_resize (GTK_WINDOW (
 							note->w_window),
@@ -827,8 +855,8 @@ stickynotes_load (GdkScreen *screen)
 			
 			/* Retrieve and set the window position of the note */
 			{
-				gchar *x_str = xmlGetProp(node, "x");
-				gchar *y_str = xmlGetProp(node, "y");
+				gchar *x_str = (gchar *)xmlGetProp(node, XML_CHAR ("x"));
+				gchar *y_str = (gchar *)xmlGetProp(node, XML_CHAR ("y"));
 				if (x_str && y_str)
 				{
 					if (atoi(x_str) != -1 ||
@@ -851,7 +879,7 @@ stickynotes_load (GdkScreen *screen)
 			{
 				char *workspace_str;
 
-				workspace_str = xmlGetProp (node, "workspace");
+				workspace_str = (gchar *)xmlGetProp (node, XML_CHAR ("workspace"));
 				if (workspace_str)
 				{
 					note->workspace = atoi (workspace_str);
@@ -862,7 +890,7 @@ stickynotes_load (GdkScreen *screen)
 			/* Retrieve and set (if any) the body contents of the
 			 * note */
 			{
-				gchar *body = xmlNodeListGetString(doc,
+				gchar *body = (gchar *)xmlNodeListGetString(doc,
 						node->xmlChildrenNode, 1);
 				if (body) {
 					GtkTextBuffer *buffer;
@@ -881,7 +909,7 @@ stickynotes_load (GdkScreen *screen)
 			/* Retrieve and set the locked state of the note,
 			 * by default unlocked */
 			{
-				gchar *locked = xmlGetProp(node, "locked");
+				gchar *locked = (gchar *)xmlGetProp(node, XML_CHAR ("locked"));
 				if (locked)
 					stickynote_set_locked(note,
 						!strcmp(locked, "true"));
@@ -902,15 +930,10 @@ stickynotes_load (GdkScreen *screen)
 	{
 		StickyNote *note = tmp1->data;
 
-		stickynote_set_visible (note, gconf_client_get_bool (
-						stickynotes->gconf,
-						GCONF_PATH "/settings/visible",
-						NULL));
-
+		stickynote_set_visible (note, TRUE);
 		tmp1 = tmp1->next;
 	}
 
-
 	g_list_free (new_notes);
 	g_list_free (new_nodes);
 
